#Tracé d'un cadran solaire d'azimut, gnomon vertical
#Le point rouge central est l'emplacement du gnomon

#Appel de modules  Python :
import matplotlib.pyplot as plt
import numpy as np

#PARAMETRES Gnomoniques -------------------------------------
lat=np.radians(49.5)
Hzero=np.arccos(-np.tan(lat)*np.tan(np.radians(23.4)))#angle horaire maximal (été)
heure0=round(np.degrees(Hzero)/15)

Rmin,  Rmax =2, 14#Définition des rayons

#Définitions de fonctions ----------------------------------------

def rayonJ(J):#fonction affine entre le rang J du jour et le rayon
    return (Rmax-Rmin)/183*J+Rmin-(Rmax-Rmin)/183*172

def azimutcorrige(A, Ah):
    if np.sin(A)<0:
        if np.sin(Ah)>0:
            A=A+np.pi
    else:
       if np.sin(Ah)<0:
        A=A-np.pi
    return A

def declinaison(J):#modélise la déclinaison pour un jour de rang J dans l'année
    return np.radians(23.44*np.sin(2*np.pi/365.24*(284+J)))

#a : longueur minimale du gnomon vertical (midi au solstice d'hiver)
a=Rmax*np.tan(np.pi/2-lat+declinaison(355))

#---------------------------------------------------------------------------------
plt.rcParams["font.family"] = "Ubuntu"#ou autre police de caractère installée sur l'ordinateur

#Paramètres du graphique --------------------------------

Lfigure=20#cm
fig=plt.figure(figsize=(Lfigure/2.54,Lfigure/2.54), tight_layout=True,  edgecolor='royalblue', linewidth=2)
ax=fig.add_subplot(111, projection='polar')
plt.text(np.radians(180),0.97*Rmax,"Cadran solaire d'azimut", fontsize=16, ha='center', bbox=dict(boxstyle=f"round", fc='white', alpha=0.5))

#Graduations d'angles :
ax.set_theta_zero_location("N")#Angle 0 en haut
listeAzimuts=np.arange(0, 360, 10)
plt.xticks(np.radians(listeAzimuts), [str(listeAzimuts[-n])+"°" for n in range(len(listeAzimuts))])

#Graduations des rayons (dates) :
ax.set_rlabel_position(5)#position des graduations des rayons
plt.ylim(0.5, Rmax+2)
initiale1=["J", "A", "S", "O", "N", "D"]#initiale du mois
listeDates=[202, 233, 265, 294, 325, 355]#rangs des 21emes jours du mois
plt.yticks([rayonJ(date) for date in listeDates], initiale1, fontsize=8)
ax.set_yticklabels(initiale1, ha='center', va='center')
initiale2=["","J", "F", "M", "A", "M"]#initiale du mois
for N,  mois in enumerate(initiale2):
    plt.text(np.radians(-5), rayonJ(listeDates[len(listeDates)-N-1]), mois, ha='center', va='center', fontsize=8)

#inscription de la latitude :
plt.text(np.pi, Rmax+1, "%s°"%(round(np.degrees(lat), 1)), fontsize=12, color='red', ha='center', va='center', fontweight='bold', bbox=dict(boxstyle=f"round", fc='white', alpha=0.5))

#---------------------------------------------------------
#Tracé des lignes horaires :
if lat<np.radians(66.66):#liste des angles horaires
    listeH=np.radians(np.arange(-heure0*15, heure0*15+15, 15))#gamme d'angle horaire
else: listeH=np.radians(np.arange(-180, 180+15, 15))

rangJours=np.arange(172, 355+0.5, 0.5)#rang du jour dans l'année, du 21 juin au 21 déc.
for Ah in listeH:
    listeAz=[]
    listeRayon=[]
    for J in rangJours:
        declin=declinaison(J)
        Ah0=np.arccos(-np.tan(lat)*np.tan(declin))
        Az=np.arctan(np.sin(Ah)/(np.sin(lat)*np.cos(Ah)-np.tan(declin)*np.cos(lat)))
        if abs(Ah)<=abs(Ah0):
            Az=azimutcorrige(Az, Ah)
            listeAz.append(Az)
            listeRayon.append(rayonJ(J))
    plt.plot(listeAz, listeRayon, color='C0')#tracé des courbes horaires
    if len(listeAz)>0:
        plt.text(listeAz[-1], Rmax+0.85, "%s"%(int(12-round(np.degrees(Ah)/15))), ha='center', va='center', bbox=dict(boxstyle=f"circle", fc='lightsteelblue'))
#---------------------------------------------------------
#Tracé des demi-heures :
if lat<np.radians(66.66):
    listeH=np.radians(np.arange(-heure0*15+7.5, heure0*15+7.5, 15))#gamme d'angle horaire
else: listeH=np.radians(np.arange(-180, 180+15, 15))
for Ah in listeH:
    listeAz=[]
    listeRayon=[]
    for J in rangJours:
        declin=declinaison(J)
        Ah0=-np.arccos(-np.tan(lat)*np.tan(declin))
        Az=np.arctan(np.sin(Ah)/(np.sin(lat)*np.cos(Ah)-np.tan(declin)*np.cos(lat)))
        if abs(Ah)<=abs(Ah0):
            Az=azimutcorrige(Az, Ah)
            listeAz.append(Az)
            listeRayon.append(rayonJ(J))
    plt.plot(listeAz, listeRayon, color='C1', alpha=0.7, lw=1)#tracé des courbes 1/2 heures
    #---------------------------------------------------------
#Tracé des 3 cercles de saisons :
DatesSaisons={"été" : 172,"équinoxes" : 265,"hiver" : 355}
for jour in DatesSaisons:
    theta=np.arange(0, 2*np.pi+0.1, 0.1)
    r=theta*0+rayonJ(DatesSaisons[jour])
    plt.plot(theta, r, 'r', linewidth=0.7)
    plt.text(3*np.pi/4, rayonJ(DatesSaisons[jour]), jour, rotation=-40, fontsize=7, ha='center', va='top')

#---------------------------------------------------------
#Tracé des limites de lever/coucher :
for Ah in np.arange(0, Hzero, 5):
    listeAz=[]
    listeRayon=[]
    for J in rangJours:
        declin=declinaison(J)
        Az0=np.arccos(-np.sin(declin)/np.cos(lat))
        Az0=azimutcorrige(Az0, Ah)
        listeAz.append(Az0)
        listeRayon.append(rayonJ(J))
    plt.plot(listeAz, listeRayon, 'g', lw='1')
    plt.plot([-A for A in listeAz], listeRayon, 'g', lw='1')
plt.text((max(listeAz)+min(listeAz))/2, rayonJ(294), 'lever', c='g', fontsize=9, ha='right')
plt.text(-(max(listeAz)+min(listeAz))/2, rayonJ(294), 'coucher', c='g', fontsize=9, ha='left')
#---------------------------------------------------------
plt.scatter(0, 0.5, s=20, c='r', zorder=3)#point rouge central
#---------------------------------------------------------
#segment du style :
plt.plot([np.radians(180), np.radians(180)], [Rmin, Rmin+a], lw='4', color='darkblue', solid_capstyle='butt')
plt.text(np.radians(173), Rmin+a/2, "style", rotation=90, ha='center', va='center')
#---------------------------------------------------------
#Indication des points cardinaux :
plt.text(0, Rmax+4, "N", ha='center')
plt.text(np.pi, Rmax+5, "S", ha='center')
#---------------------------------------------------------
#ombre :
azimutOmbre=np.radians(-60)
#plt.plot([azimutOmbre, azimutOmbre], [0.5, Rmax+2], lw='4', color='darkgray',alpha=0.7, zorder=3)
#---------------------------------------------------------
#Tracé d'un mur :
declinMur=np.radians(20)#positive vers l'Ouest
#plt.plot([-declinMur+np.pi/2, -declinMur+np.pi+np.pi/2], [Rmax, Rmax], c='orangered', zorder=3)
#---------------------------------------------------------
NomFichier="CadranAzimut"+str(round(np.degrees(lat), 1))
fig.savefig(NomFichier+".png", dpi=200)# Création image PNG
fig.savefig(NomFichier+".pdf")# Création  PDF
#---------------------------------------------------------
plt.show()
